#include <Adafruit_GFX.h>         // Core graphics library
#include <Adafruit_ST7735.h>      // Hardware-specific library
#include <SdFat.h>                // SD card & FAT filesystem library
#include <Adafruit_SPIFlash.h>    // SPI / QSPI flash library
#include <Adafruit_ImageReader.h> // Image-reading functions
#include <Wire.h>
#include <SimpleDHT.h>
#include <Adafruit_Sensor.h>  // include Adafruit sensor library
#include <Adafruit_BMP280.h>  // include adafruit library for BMP280 sensor

int sensorPin=47;
int sound_cnt = 0;
boolean val = 0;

int photocellPin = 1;     // the cell and 10K pulldown are connected to a0
int photocellReading;     // the analog reading from the sensor divider

// Comment out the next line to load from SPI/QSPI flash instead of SD card:
#define USE_SD_CARD

// TFT display and SD card share the hardware SPI interface, using
// 'select' pins for each to identify the active device on the bus.

#define SD_CS   53 // SD card select pin

#define TFT_cs         4
#define TFT_sclk       5
#define TFT_mosi       6
#define TFT_dc         7  // Or set to -1 and connect to Arduino RESET pin
#define TFT_rst        8

#define BLACK    0x0000
#define BLUE     0x001F
#define RED      0xF800
#define GREEN    0x07E0
#define CYAN     0x07FF
#define MAGENTA  0xF81F
#define YELLOW   0xFFE0 
#define WHITE    0xFFFF

#if defined(USE_SD_CARD)
  SdFat                SD;         // SD card filesystem
  Adafruit_ImageReader reader(SD); // Image-reader object, pass in SD filesys
#else
  // SPI or QSPI flash filesystem (i.e. CIRCUITPY drive)
  #if defined(__SAMD51__) || defined(NRF52840_XXAA)
    Adafruit_FlashTransport_QSPI flashTransport(PIN_QSPI_SCK, PIN_QSPI_CS,
      PIN_QSPI_IO0, PIN_QSPI_IO1, PIN_QSPI_IO2, PIN_QSPI_IO3);
  #else
    #if (SPI_INTERFACES_COUNT == 1)
      Adafruit_FlashTransport_SPI flashTransport(SS, &SPI);
    #else
      Adafruit_FlashTransport_SPI flashTransport(SS1, &SPI1);
    #endif
  #endif
  Adafruit_SPIFlash    flash(&flashTransport);
  FatFileSystem        filesys;
  Adafruit_ImageReader reader(filesys); // Image-reader, pass in flash filesys
#endif

Adafruit_ST7735      tft    = Adafruit_ST7735(TFT_cs, TFT_dc, TFT_mosi, TFT_sclk, TFT_rst);
Adafruit_Image       img;        // An image loaded into RAM
int32_t              width  = 0, // BMP image dimensions
                     height = 0;
ImageReturnCode stat; // Status from image-reading functions

// Settings of DS3231
#define DS3231_I2C_ADDRESS 104
 
byte seconds, minutes, hours, day, date, month, year;
byte tMSB, tLSB;

int pinDHT22 = 49;
SimpleDHT22 dht22(pinDHT22);
byte temperature = 0;
byte humidity = 0;

File myFile;
String file_name_log;

#define BMP280_I2C_ADDRESS  0x76
Adafruit_BMP280 bmp280;

void setup(void) {

  Wire.begin();
  Serial.begin(9600);
  tft.initR(INITR_BLACKTAB); // Initialize screen

  // The Adafruit_ImageReader constructor call (above, before setup())
  // accepts an uninitialized SdFat or FatFileSystem object. This MUST
  // BE INITIALIZED before using any of the image reader functions!
  Serial.print(F("Initializing filesystem..."));
#if defined(USE_SD_CARD)
  // SD card is pretty straightforward, a single call...
  if(!SD.begin(SD_CS, SD_SCK_MHZ(10))) { // Breakouts require 10 MHz limit due to longer wires
    Serial.println(F("SD begin() failed"));
    for(;;); // Fatal error, do not continue
  }
#else
  // SPI or QSPI flash requires two steps, one to access the bare flash
  // memory itself, then the second to access the filesystem within...
  if(!flash.begin()) {
    Serial.println(F("flash begin() failed"));
    for(;;);
  }
  if(!filesys.begin(&flash)) {
    Serial.println(F("filesys begin() failed"));
    for(;;);
  }
#endif
  Serial.println(F("OK!"));

  // Fill screen blue. Not a required step, this just shows that we're
  // successfully communicating with the screen.
  tft.setRotation(1);
  tft.fillScreen(ST7735_BLACK);

  // randomSeed() will then shuffle the random function.
  randomSeed(analogRead(0));

  // 대기압 센서 시작
  if (!bmp280.begin(BMP280_I2C_ADDRESS)) {  
    Serial.println("Could not find a valid BMP280 sensor, check wiring!");
    while (1);
  }
 
  // 새로운 파일 생성
  get3231Date();
  get_filename();
  myFile = SD.open(file_name_log, FILE_WRITE);

  // Sound 센서 입력포트
  pinMode(sensorPin, INPUT);
}

int selection = 0;
void loop() {
  String temp_str = "";
  String hum_str = "";

  String today_str = "";
  String day_in_week = "";
  
  String file_name = "";
  String tm_stmp = "";
  int randNumber = 0;

  float t_pressure = 0;  
  String air_pressure = "";
  String home_height = "";
  String other_temp = "";

  // get temperature, pressure and altitude from library
  float BMP_temp = bmp280.readTemperature();  // get temperature
  float pressure    = bmp280.readPressure();     // get pressure
  float altitudes   = bmp280.readAltitude(1013.25); // get altitude (this should be adjusted to your local forecast)

  // CDS reading
  String CDS_val = "";
  
  val = digitalRead(sensorPin);
  // Serial.println (val);
  randNumber = random(0, 400);
  if (randNumber < 10) {
    file_name = "BMP/00"+(String) randNumber+".bmp"; 
  }
  else if (randNumber >= 10 && randNumber < 100) {
    file_name = "BMP/0"+(String) randNumber+".bmp";
  }
  else {
    file_name = "BMP/"+(String) randNumber+".bmp";
  }
  stat = reader.drawBMP(&file_name[0], tft, 0, 0);
  reader.printStatus(stat);   // How'd we do? 

  for (int j=0;j<20;j++) {
    val = digitalRead(sensorPin);
    tft.setCursor(21, 114);
    tft.setTextColor(BLACK, WHITE);
    tft.setTextSize(2);
    tm_stmp = Get_time_stamp();
    tft.print(tm_stmp); 
    delay(500);
  }
  DHT22_read();
  temp_str = (String) temperature+"*C ";
  hum_str = (String) humidity+"%RH ";

  // 대기압 센서 가동
  t_pressure = pressure / 100;
  other_temp = (String) BMP_temp + "*C";
  air_pressure = (String) t_pressure + "hPa";
  home_height = (String) altitudes + "m";

  // CDS Reading
  photocellReading = analogRead(photocellPin); 
  Serial.print("Analog reading = ");
  Serial.println(photocellReading);     // the raw analog reading
  CDS_val = (String) photocellReading;

  switch (selection) {
    case 0:
      Show_temp_hum(temp_str, hum_str);
      selection += 1;
      break;
    case 1:
      Show_dates();
      selection += 1;
      break;
    case 2:
      Show_pressure(other_temp, air_pressure, home_height);
      selection = 0;
      break;
  }
  
  // Writing log to file
  switch (day) {
    case 1:
      day_in_week = "Sunday";
      break;
    case 2:
      day_in_week = "Monday";
      break;
    case 3:
      day_in_week = "Tuseday";
      break;
    case 4:
      day_in_week = "Wendsday";
      break;
    case 5:
      day_in_week = "Thursday";
    case 6:
      day_in_week = "Friday";
      break;
    case 7:
      day_in_week = "Saturday";
      break;
  }

  if (val==HIGH) {
    sound_cnt += 1;
    val = LOW;  
  }
  
  today_str = "20"+(String) year+"-"+(String) month+"-"+(String) date+" "+(String) day_in_week+" ";
  get_filename();
  if (!SD.exists(&file_name_log[0])) {
    myFile = SD.open(file_name_log, FILE_WRITE);
    sound_cnt = 0;  
  }
  myFile.print(today_str);
  myFile.print(tm_stmp);
  myFile.print(temp_str);
  myFile.print(hum_str);
  myFile.print(" "+other_temp);
  myFile.print(" "+air_pressure);
  myFile.print(" "+home_height);
  myFile.print(" "+(String) sound_cnt);
  myFile.println(" "+CDS_val);
  myFile.close();
  myFile = SD.open(file_name_log, FILE_WRITE);  
}

void Show_pressure(String o_temp, String a_pressure, String h_height) {
  // print data on the serial monitor software
  /*e
  Serial.print("Temperature = ");
  Serial.print(temperature);
  Serial.println(" °C");
  // 2: print pressure
  Serial.print("Pressure    = ");
  Serial.print(pressure/100);
  Serial.println(" hPa");
  // 3: print altitude
  Serial.print("Approx Altitude = ");
  Serial.print(altitude_);
  Serial.println(" m");
  */
  
  tft.fillScreen(ST7735_BLACK);
  tft.drawRect(1, 1, 159, 127, YELLOW);
  tft.setCursor(5, 5);
  tft.setTextColor(WHITE, BLACK);
  
  // Print temperature
  tft.setTextSize(1);
  tft.print("Temperature (measured 2)");
  tft.setCursor(10, 20); 
  tft.setTextSize(2);
  tft.setTextColor(RED, BLACK);
  tft.print(o_temp);

  // Print pressure
  tft.setTextColor(WHITE, BLACK);
  tft.setTextSize(1);
  tft.setCursor(5, 45);
  tft.print("Present air pressure");
  tft.setCursor(10, 60); 
  tft.setTextSize(2);
  tft.setTextColor(YELLOW, BLACK);
  tft.print(a_pressure);

  // Print altitude
  tft.setTextColor(WHITE, BLACK);
  tft.setTextSize(1);
  tft.setCursor(5, 85);
  tft.print("Altitude of this place");
  tft.setCursor(10, 100); 
  tft.setTextSize(2);
  tft.setTextColor(GREEN, BLACK);
  tft.print(h_height);  
 
  delay(5000);
  tft.fillScreen(ST7735_BLACK);
}

void get_filename() {
  String t_month = "";
  String t_day = "";
  
  if (month < 10) {
    t_month = "0"+(String) month;
  }
  else {
    t_month = (String) month;
  }
  
  if (date < 10) {
    t_day = "0"+(String) date;
  }
  else {
    t_day = (String) date;
  }
  file_name_log = "D"+(String) year+(String) t_month+(String) t_day+".txt";    
}

// byte seconds, minutes, hours, day, date, month, year;
void Show_dates() {
  String today_is = "";
  String day_in_week = "";
  
  today_is += (String) year+"/";
  today_is += (String) month+"/";
  today_is += (String) date;
  
  tft.fillScreen(ST7735_BLACK);
  tft.drawRect(1, 1, 159, 127, RED);
  tft.setCursor(5, 12);
  tft.setTextColor(WHITE, BLACK);
  
  // Print dates in year/month/day
  tft.setTextSize(2);
  tft.print("Today is");
  tft.setCursor(5, 35); 
  tft.setTextSize(3);
  tft.setTextColor(YELLOW, BLACK);
  tft.print(today_is);

  // Print humidity
  tft.setTextColor(WHITE, BLACK);
  tft.setTextSize(2);
  tft.setCursor(5, 72);
  tft.print("Weekday is");
  tft.setCursor(5, 95); 
  tft.setTextSize(3);
  tft.setTextColor(BLUE, BLACK);
  // Day in a week(1~7, Sunday1 Monday2 Tuseday3 Wendsday4 Thursday5 Friday6 Saturday7)
  
  switch (day) {
    case 1:
      day_in_week = "Sunday";
      break;
    case 2:
      day_in_week = "Monday";
      break;
    case 3:
      day_in_week = "Tuseday";
      break;
    case 4:
      day_in_week = "Wendsday";
      break;
    case 5:
      day_in_week = "Thursday";
      break;
    case 6:
      day_in_week = "Friday";
      break;
    case 7:
      day_in_week = "Saturday";
      break;
  }   
  tft.print(day_in_week);
  
  delay(5000);
  tft.fillScreen(ST7735_BLACK);  
}

void Show_temp_hum(String temp, String hum) {
  tft.fillScreen(ST7735_BLACK);
  tft.drawRect(1, 1, 159, 127, GREEN);
  tft.setCursor(5, 12);
  tft.setTextColor(WHITE, BLACK);
  
  // Print temperature
  tft.setTextSize(2);
  tft.print("Temperature");
  tft.setCursor(15, 35); 
  tft.setTextSize(3);
  tft.setTextColor(RED, BLACK);
  tft.print(temp);

  // Print humidity
  tft.setTextColor(WHITE, BLACK);
  tft.setTextSize(2);
  tft.setCursor(5, 72);
  tft.print("Humidity");
  tft.setCursor(15, 95); 
  tft.setTextSize(3);
  tft.setTextColor(YELLOW, BLACK);
  tft.print(hum);
 
  delay(5000);
  tft.fillScreen(ST7735_BLACK);
}

void DHT22_read() {
  int err = SimpleDHTErrSuccess;
  if ((err = dht22.read(&temperature, &humidity, NULL)) != SimpleDHTErrSuccess) {
//    Serial.print("Read DHT22 failed, err="); Serial.println(err);delay(2000);
    return;
  }
}

String Get_time_stamp() {
  String time_stamp = "";
  String hrs = "";
  String mins = "";
  String secs = "";
  
  watchConsole();
  get3231Date();

  if (hours < 10) {
    hrs = "0"+(String) hours;
  }
  else {
    hrs = (String) hours;
  }
  if (minutes < 10) {
    mins = "0"+(String) minutes;
  }
  else {
    mins = (String) minutes;
  }
  if (seconds < 10) {
    secs = "0"+(String) seconds;
  }
  else {
    secs = (String) seconds;
  }
  time_stamp = " "+hrs+":"+mins+":"+secs+" ";
  return (time_stamp);
}
 
// Convert decimal to Binary Coded Decimal
byte decToBcd(byte val)
{
  return ( (val/10*16) + (val%10) );
}
 
void watchConsole() {
  if (Serial.available()) {      // Look for char in serial queue and process if found
    if (Serial.read() == 84) {   //If command = "T" Set Date
      set3231Date();
      get3231Date();
      Serial.println(" ");
    }
  }
}
 
// Time setting
// T(Set command) + Year(00~99) + Month(01~12) + Day(01~31) + Hour(00~23) + Minute(00~59) + Second(00~59) + Day in a week(1~7, Sunday1 Monday2 Tuseday3 Wendsday4 Thursday5 Friday6 Saturday7)
// Example: T2104171739007 (Year 2021 April day 17th 13 hour 00 minutes 00 second Saturday)
void set3231Date() {
  year    = (byte) ((Serial.read() - 48) *10 +  (Serial.read() - 48));
  month   = (byte) ((Serial.read() - 48) *10 +  (Serial.read() - 48));
  date    = (byte) ((Serial.read() - 48) *10 +  (Serial.read() - 48));
  hours   = (byte) ((Serial.read() - 48) *10 +  (Serial.read() - 48));
  minutes = (byte) ((Serial.read() - 48) *10 +  (Serial.read() - 48));
  seconds = (byte) ((Serial.read() - 48) * 10 + (Serial.read() - 48));
  day     = (byte) (Serial.read() - 48);
 
  Wire.beginTransmission(DS3231_I2C_ADDRESS);
  Wire.write(0x00);
  Wire.write(decToBcd(seconds));
  Wire.write(decToBcd(minutes));
  Wire.write(decToBcd(hours));
  Wire.write(decToBcd(day));
  Wire.write(decToBcd(date));
  Wire.write(decToBcd(month));
  Wire.write(decToBcd(year));
  Wire.endTransmission();
}
  
void get3231Date() {
  // send request to receive data starting at register 0
  Wire.beginTransmission(DS3231_I2C_ADDRESS); // 104 is DS3231 device address
  Wire.write(0x00); // start at register 0
  Wire.endTransmission();
  Wire.requestFrom(DS3231_I2C_ADDRESS, 7); // request seven bytes
 
  if(Wire.available()) {
    seconds = Wire.read(); // get seconds
    minutes = Wire.read(); // get minutes
    hours   = Wire.read();   // get hours
    day     = Wire.read();
    date    = Wire.read();
    month   = Wire.read(); //temp month
    year    = Wire.read();
       
    seconds = (((seconds & B11110000)>>4)*10 + (seconds & B00001111)); // convert BCD to decimal
    minutes = (((minutes & B11110000)>>4)*10 + (minutes & B00001111)); // convert BCD to decimal
    hours   = (((hours & B00110000)>>4)*10 + (hours & B00001111)); // convert BCD to decimal (assume 24 hour mode)
    day     = (day & B00000111); // 1-7
    date    = (((date & B00110000)>>4)*10 + (date & B00001111)); // 1-31
    month   = (((month & B00010000)>>4)*10 + (month & B00001111)); //msb7 is century overflow
    year    = (((year & B11110000)>>4)*10 + (year & B00001111));
  }
  else {
    //oh noes, no data!
  }
}
